#include "hotkeylistener.h"

/*
 * Due to the way Qt is designed, we have to have a GUI application in order
 * to even consider listening for keyboard shortcuts. Qxt respects this, so
 * their global shortcut class requires a window. So, we attempt to do the
 * the best we can to keep the fact that we're technically a GUI app hidden.
 */
HotKeyListener::HotKeyListener(QObject *parent) : QObject(parent)
{
    // Create a window that will forever remain hidden and useless.
    // This is a requirement of Qt's Shortcut handling.
    QWidget mainWindow;
    mainWindow.setWindowFlags(Qt::Tool);

    QxtGlobalShortcut* play = new QxtGlobalShortcut(this);
    connect(play, SIGNAL(activated()), this, SLOT(fireShortcut()));
    play->setShortcut(QKeySequence("Ctrl+F9"));
    shortcuts[play] = QString("Play");

    QxtGlobalShortcut* next = new QxtGlobalShortcut(this);
    connect(next, SIGNAL(activated()), this, SLOT(fireShortcut()));
    next->setShortcut(QKeySequence("Ctrl+F12"));
    shortcuts[next] = QString("Next");

    QxtGlobalShortcut* prev = new QxtGlobalShortcut(this);
    connect(prev, SIGNAL(activated()), this, SLOT(fireShortcut()));
    prev->setShortcut(QKeySequence("Ctrl+F11"));
    shortcuts[prev] = QString("Prev");

    // Attempt to connect to GMusic
    localSocket = new QLocalSocket();

    // Finish Initialization
    connectSignals();

    // Last step is attempting to connect to the server
    localSocket->connectToServer("gmusic");
} // end HotKeyListener

HotKeyListener::~HotKeyListener()
{
    localSocket->close();
    localSocket->deleteLater();
} // end ~HotKeyListener

void HotKeyListener::connectSignals()
{
    connect(localSocket, SIGNAL(connected()), this, SLOT(handleConnected()));
    connect(localSocket, SIGNAL(error(QLocalSocket::LocalSocketError)), this, SLOT(handleError(QLocalSocket::LocalSocketError)));
    connect(localSocket, SIGNAL(readyRead()), this, SLOT(incomingData()));
} // end connectSignals

void HotKeyListener::fireShortcut()
{
    QxtGlobalShortcut* qxtShortcut = qobject_cast<QxtGlobalShortcut*>(QObject::sender());
    QString shortcut = shortcuts[qxtShortcut];

    printf("%s Fired!\n", qPrintable(shortcut));
    fflush(stdout);

    if (shortcut == "Play" && connected)
    {
       localSocket->write("4:play,");
    }
    else if (shortcut == "Prev" && connected)
    {
       localSocket->write("4:prev,");
    }
    else if (shortcut == "Next" && connected)
    {
       localSocket->write("4:next,");
    } // end if
} // end fireShortcut

void HotKeyListener::handleConnected()
{
    connected = true;

    printf("Connected!\n");
    fflush(stdout);

    //localSocket->write("4:echo,");
} // end handleConnected

void HotKeyListener::handleError(QLocalSocket::LocalSocketError error)
{
    printf("Socket encoutered an error: %s\n", qPrintable(localSocket->errorString()));
    fflush(stdout);
} // end handleError

void HotKeyListener::incomingData()
{
    QString data = QString(localSocket->readAll());
    incomingDataBuffer.append(data);

    printf("Incoming Data: %s\n", qPrintable(data));
    fflush(stdout);

    bool success;
    int numBytes, nextSeparatorPos, commaPos;
    QByteArray numBytesString, tempData;

    while(not incomingDataBuffer.isEmpty())
    {
        nextSeparatorPos = incomingDataBuffer.indexOf(':');
        if(nextSeparatorPos == -1)
        {
            // We don't have enough characters yet to parse this field.
            return;
        } // end if

        numBytesString = incomingDataBuffer.left(nextSeparatorPos);
        numBytes = numBytesString.toInt(&success, 10);
        if(not success)
        {
            commaPos = numBytesString.indexOf(',');
            if(commaPos == -1)
            {
                printf("Couldn't determine message length! Dropping current buffer contents.\n");
                fflush(stdout);
                //log->error("Couldn't determine message length! Dropping current buffer contents.");
                incomingDataBuffer.clear();
                return;
            }
            else
            {
                numBytes = numBytesString.right(numBytesString.length() - commaPos - 1).toLongLong(&success, 36);
                if(not success)
                {
                    printf("Couldn't determine message length! Dropping current buffer contents.\n");
                    fflush(stdout);
                    //log->error("Couldn't determine message length! Dropping current buffer contents.");
                    incomingDataBuffer.clear();
                    return;
                }
                else
                {
                    printf("Ignoring garbage data before message length specifier!\n");
                    fflush(stdout);
                    //log->warning("Ignoring garbage data before message length specifier!");
                } // end if
            } // end if
        } // end if

        if(incomingDataBuffer.length() < (nextSeparatorPos + numBytes + 2))
        {
            // We don't have enough characters yet to parse this field.
            break;
        } // end if

        incomingDataBuffer.remove(0, nextSeparatorPos + 1);

        // Ensure that the netstring ends with a ','.
        if(incomingDataBuffer.at(numBytes) != ',')
        {
            printf("Incorrectly formatted netstring!\n");
            fflush(stdout);
            //log->error("Incorrectly formatted netstring!");
            break;
        } // end if

        tempData = incomingDataBuffer.left(numBytes);
        incomingDataBuffer.remove(0, numBytes + 1);

        printf("Got Command: %s\n", tempData.constData());
        fflush(stdout);

        emit gotCommand(QString(tempData));
    } // end while
} // end incomingData
