# -*- coding: utf-8 -*-

# Copyright (c) 2011 Christopher S. Case and David H. Bronke
# Licensed under the MIT license; see the LICENSE file for details.

# Python imports
import os
import logging
import urllib

# Import python qt modules
from Qt import QtCore, QtGui, Signal

# Classes imports
from classes.common import MessageTypes, Instances
from classes.playerpage import PlayerWindow
from classes.remotelistener import RemoteListener
from classes.notify import Notify


logger = logging.getLogger("gmusic.main")


class GMain(QtCore.QObject):
    """The main GMusic class. Contains the system tray, and all the control
    signals. Handled registering the global hotkeys. All other windows should
    have a reference to this object.

    """
    #--------------------------------------------------------------------------#
    # Signals
    #--------------------------------------------------------------------------#
    play = Signal()
    stop = Signal()
    next = Signal()
    prev = Signal()
    playlist = Signal(str)

    #--------------------------------------------------------------------------#
    # Initialization
    #--------------------------------------------------------------------------#
    def __init__(self):
        QtCore.QObject.__init__(self)

        # Set our global instances
        Instances.gmain = self

        # Configure our application
        self.app = QtGui.QApplication.instance()
        self.app.setQuitOnLastWindowClosed(False)

        # Instantiate our listener for remote controllers
        self.remoteListener = RemoteListener()

        # Setup notification support
        self.notifier = Notify("GMusic")
        Instances.notifier = self.notifier

        # Finish init
        self.createWidgets()
        self.connectSignals()

    def createWidgets(self):
        logger.debug("Creating widgets")

        # Setup the system tray icon
        self.resourcedir = os.path.join(
            QtGui.QApplication.instance().property("AppPath"), "resources")

        # Setup tray icon
        self.icon = QtGui.QIcon(self.resourcedir + '/gMusic_16.png')
        self.trayIcon = QtGui.QSystemTrayIcon(self.icon)

        # Setup the player window
        self.playerWindow = PlayerWindow()

        # Setup the tray icon context menu
        self.trayMenu = QtGui.QMenu()
        self.openAction = QtGui.QAction(QtGui.QIcon(self.resourcedir + '/open_16.png'),
                'Open Player', self)
        self.trayMenu.addAction(self.openAction)
        self.trayMenu.addSeparator()

        self.playAction = QtGui.QAction(QtGui.QIcon(self.resourcedir + '/play_16.png'),
                '&Play', self)
        self.playAction.setShortcut(QtGui.QKeySequence
            ("Ctrl+F9"))
        self.playShortcut = QtGui.QShortcut(QtGui.QKeySequence
            ("Ctrl+F9"), self.playerWindow)
        self.playShortcut.setContext(QtCore.Qt.ApplicationShortcut)
        self.trayMenu.addAction(self.playAction)

        self.nextAction = QtGui.QAction(QtGui.QIcon(self.resourcedir + '/next_16.png'),
                '&Next', self)
        self.nextAction.setShortcut(QtGui.QKeySequence
            ("Ctrl+F12"))
        self.nextShortcut = QtGui.QShortcut(QtGui.QKeySequence
            ("Ctrl+F12"), self.playerWindow)
        self.nextShortcut.setContext(QtCore.Qt.ApplicationShortcut)
        self.trayMenu.addAction(self.nextAction)

        self.prevAction = QtGui.QAction(QtGui.QIcon(self.resourcedir + '/prev_16.png'),
                'P&revious', self)
        self.prevAction.setShortcut(QtGui.QKeySequence
            ("Ctrl+F11"))
        self.prevShortcut = QtGui.QShortcut(QtGui.QKeySequence
            ("Ctrl+F11"), self.playerWindow)
        self.prevShortcut.setContext(QtCore.Qt.ApplicationShortcut)
        self.trayMenu.addAction(self.prevAction)
        self.trayMenu.addSeparator()

        self.playlistMenu = self.trayMenu.addMenu("Playlists")
        self.trayMenu.addSeparator()

        self.optionsAction = QtGui.QAction(
                QtGui.QIcon(self.resourcedir + '/settings_16.png'), '&Options', self)
        self.trayMenu.addAction(self.optionsAction)
        self.trayMenu.addSeparator()

        self.exitAction = QtGui.QAction(QtGui.QIcon(self.resourcedir + '/exit.png'),
                                        'E&xit', self)
        self.trayMenu.addAction(self.exitAction)

        # Add menu to tray icon
        self.trayIcon.setContextMenu(self.trayMenu)

        # Start Remote server
        self.remoteListener.start()

    def connectSignals(self):
        # Tray Icon Signals
        self.trayIcon.activated.connect(self.handleTrayActivated)

        # Player signals
        self.play.connect(self.playerWindow.handlePlayPause)
        self.prev.connect(self.playerWindow.handlePrev)
        self.next.connect(self.playerWindow.handleNext)
        self.playlist.connect(self.playerWindow.handleSelectPlaylist)

        self.playerWindow.infoUpdated.connect(self.handleInfoUpdated)
        self.playerWindow.trackChanged.connect(self.handleTrackChanged)

        # Shortcuts
        self.playShortcut.activated.connect(self.handlePlayPauseAction)
        self.nextShortcut.activated.connect(self.handleNextAction)
        self.prevShortcut.activated.connect(self.handlePrevAction)

        # Actions
        self.playAction.triggered.connect(self.handlePlayPauseAction)
        self.nextAction.triggered.connect(self.handleNextAction)
        self.prevAction.triggered.connect(self.handlePrevAction)
        self.openAction.triggered.connect(self.handleOpenAction)
        self.optionsAction.triggered.connect(self.handleOptionsAction)
        self.exitAction.triggered.connect(self.handleExitAction)

    #--------------------------------------------------------------------------#
    # Helpers
    #--------------------------------------------------------------------------#

    def show(self):
        logger.debug("Showing tray icon")
        self.trayIcon.show()

    def getStatus(self):
        return self.playerWindow.checkStatus()

    #--------------------------------------------------------------------------#
    # Slots
    #--------------------------------------------------------------------------#

    def handleTrayActivated(self, reason):
        if reason == QtGui.QSystemTrayIcon.DoubleClick:
            if self.playerWindow.isHidden():
                self.playerWindow.show()
            else:
                self.playerWindow.hide()

    def notify(self, title, message, icon=None):
        if self.notifier.notifier is not None:
            self.notifier.notify(title, message, icon)
        else:
            self.showMessage(title, message)

    def showMessage(self, title, message, type=MessageTypes.None,
            timeout=10000):
        """Displays 'message' through the tray icon's showMessage function,
        with title 'title'. 'type' is one of the enumerations of
        'common.MessageTypes'.

        """
        if type == MessageTypes.None:
            icon = QtGui.QSystemTrayIcon.NoIcon

        elif type == MessageTypes.Info:
            icon = QtGui.QSystemTrayIcon.Information

        elif type == MessageTypes.Warning:
            icon = QtGui.QSystemTrayIcon.Warning

        elif type == MessageTypes.Critical:
            icon = QtGui.QSystemTrayIcon.Critical

        self.trayIcon.showMessage(title, message, icon, timeout)

    def handlePlayPauseAction(self):
        logger.debug('Handle Play')
        self.play.emit()

        # Change Icon and Text
        if self.playAction.text() == '&Play':
            self.playAction.setText('&Pause')
            self.playAction.setIcon(QtGui.QIcon(self.resourcedir + '/pause_16.png'))
        elif self.playAction.text() == '&Pause':
            self.playAction.setText('&Play')
            self.playAction.setIcon(QtGui.QIcon(self.resourcedir + '/play_16.png'))
        else:
            logger.warn("Could not determine playAction text!")

    def handlePrevAction(self):
        logger.debug('Handle Prev')
        self.prev.emit()

    def handleNextAction(self):
        logger.debug('Handle Next')
        self.next.emit()

    def handleOpenAction(self):
        self.playerWindow.show()

    def handleOptionsAction(self):
        logger.debug("This is where we would open the options menu.")

    def handleExitAction(self):
        logger.info("Thank you for using GMusic!")
        self.playerWindow.close()
        self.app.quit()

    def handleInfoUpdated(self):
        pass

    def handleTrackChanged(self):
        artist = self.playerWindow.info['artist']
        track = self.playerWindow.info['track']
        duration = self.playerWindow.info['totalTime']

        try:
            artURL = self.playerWindow.info['art']
            artFilename = urllib.urlretrieve(artURL)[0]

        except:
            artFilename = ''

        if duration.strip() != '':
            messageString = "%s - %s (%s)" % (artist, track, duration)
        else:
            messageString = "%s - %s" % (artist, track)

        self.notify('Now Playing', messageString, artFilename)
