# -*- coding: utf-8 -*-

# Copyright (c) 2009-2011 Christopher S. Case and David H. Bronke
# Licensed under the MIT license; see the LICENSE file for details.

"""Coconut's custom QNetworkAccessManager implementation.

This gives us the ability to block arbitrary URLs using regular expressions.

Some code taken from the Arora project. (http://code.google.com/p/arora/)

"""
import re

from Qt import QtCore, QtNetwork

import networkcookiejar


networkAccessManager = None
onAuthRequired = lambda x, y: None


class BlockedNetworkReply(QtNetwork.QNetworkReply):

    def __init__(self, request, regex, parent=None):
        QtNetwork.QNetworkReply.__init__(self, parent)
        self.setOperation(QtNetwork.QNetworkAccessManager.GetOperation)
        self.setRequest(request)
        self.setUrl(request.url())
        self.setError(QtNetwork.QNetworkReply.ContentAccessDenied,
                "Blocked by ignoredUrls setting: " + str(regex))
        QtCore.QTimer.singleShot(0, self.delayedFinished)

    def readData(self, data, maxSize):
        return -1

    def writeData(self, data, maxSize):
        return -1

    def delayedFinished(self):
        self.error.emit(QtNetwork.QNetworkReply.ContentAccessDenied)
        self.finished.emit()

    def abort(self):
        pass


class NetworkAccessManager(QtNetwork.QNetworkAccessManager):
    _settings = {
            "useProxy": False,
            "proxyType": "Socks5Proxy",
            "proxyAddress": "",
            "proxyPort": 8080,
            "proxyUser": "",
            "proxyPassword": "",
            "ignoredUrls": [],
            }

    def __init__(self, parent=None):
        QtNetwork.QNetworkAccessManager.__init__(self, parent)

        self.downloads = list()

        self.ignored = list()
        for ignored in self._settings["ignoredUrls"]:
            urlRegex = ignored["ignoredUrl"].getValue()
            self.ignored.append(re.compile(urlRegex))

        if self._settings["useProxy"]:
            proxy = QtNetwork.QNetworkProxy(
                    getattr(QtNetwork.QNetworkProxy,
                        self._settings["proxyType"],
                        QtNetwork.QNetworkProxy.NoProxy),
                    self._settings["proxyAddress"],
                    self._settings["proxyPort"],
                    self._settings["proxyUser"],
                    self._settings["proxyPassword"])
        else:
            proxy = QtNetwork.QNetworkProxy()

        self.setProxy(proxy)

    def testForBlockedUrl(self, request):
        if request.url().scheme() == "data":
            return None

        for urlRegex in self.ignored:
            if urlRegex.search(str(request.url().toEncoded())) is not None:
                print "Blocked request: URL '%s' blocked by settings." \
                        % (request.url().toEncoded(), )
                return BlockedNetworkReply(request, urlRegex, self)

        return None

    def createRequest(self, op, req, outgoingData=None):
        reply = None

        if op == QtNetwork.QNetworkAccessManager.GetOperation:
            reply = self.testForBlockedUrl(req)

        if reply is None:
            reply = super(NetworkAccessManager, self).createRequest(
                    op, req, outgoingData)

        return reply


class NetworkThread(QtCore.QThread):
    def __init__(self, parent):
        self.sessionReady = QtCore.QMutex()
        self.sessionReady.lock()
        self.session = None
        self.parent = parent

        QtCore.QThread.__init__(self)

    def run(self):
        global onAuthRequired

        #FIXME: This may cause thread-exit issues, since the
        # NetworkAccessManager object won't be deleted until after the thread.
        # (See http://doc.trolltech.com/4.7.old/threads-qobject.html)
        #self.session = QtNetwork.QNetworkAccessManager()
        self.session = NetworkAccessManager(self.parent)
        self.session.setCookieJar(networkcookiejar.getCookieJar())

        #TODO: New-style connection?
        self.session.connect(self.session,
                QtCore.SIGNAL(
                    "authenticationRequired(QNetworkReply*, QAuthenticator*)"),
                onAuthRequired)

        self.sessionReady.unlock()

        self.exec_()

    def getSession(self):
        self.sessionReady.lock()
        session = self.session
        self.sessionReady.unlock()
        return session


THREADS = False

netThreads = list()


def setOnAuthRequired(onAuthenticationRequired):
    global onAuthRequired
    onAuthRequired = onAuthenticationRequired


def createSession(parent=None):
    #print "Creating new NetworkAccessManager."

    if THREADS:
        global netThreads

        namThread = NetworkThread(parent)
        netThreads.append(namThread)
        namThread.start()
        return namThread.getSession()

    else:
        session = NetworkAccessManager(parent)
        session.setCookieJar(networkcookiejar.getCookieJar())

        #TODO: New-style connection?
        session.connect(session,
                QtCore.SIGNAL(
                    "authenticationRequired(QNetworkReply*, QAuthenticator*)"),
                onAuthRequired)
        return session


def getSession():
    global networkAccessManager

    if networkAccessManager is None:
        networkAccessManager = createSession()

    return networkAccessManager
