# -*- coding: utf-8 -*-

# Copyright (c) 2011 Christopher S. Case and David H. Bronke
# Licensed under the MIT license; see the LICENSE file for details.

# Python Imports
import logging
import urllib

# Import python qt modules
from Qt import QtCore, QtGui, QtWebKit, Signal

# Instances
from classes.common import Instances

# Classes imports
import classes.networkaccessmanager as networkaccessmanager
import classes.networkcookiejar as networkcookiejar


logger = logging.getLogger("gmusic.playerpage")


class PlayerWindow(QtGui.QMainWindow):
    """

    """
    infoUpdated = Signal()
    trackChanged = Signal()
    info = None
    firstPlay = True

    def __init__(self, *args, **kwargs):
        QtGui.QMainWindow.__init__(self, *args, **kwargs)

        # Set Window Title
        self.setWindowTitle("GMusic Player")

        # Set Mac LooknFeel settings
        #self.setAttribute(QtCore.Qt.WA_MacBrushedMetal, True)


        # Set default Window size
        self.resize(800, 600)

        # Set up a timer
        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.updateInformation)
        self.timer.setInterval(1000)
        self.timer.start()

        # Get a reference to gmain
        self.gmain = Instances.gmain

        # Set the window icon
        self.setWindowIcon(QtGui.QIcon(self.gmain.resourcedir + "/gMusic_16.png"))

        # Finish initialization
        self.createWidgets()

    def createWidgets(self):
        self.webView = QtWebKit.QWebView()
        self.webView.loadFinished.connect(self.handleLoadFinished)

        self.webView.page().setNetworkAccessManager(
                networkaccessmanager.createSession(self))

        self.webView.load(QtCore.QUrl("http://music.google.com/music/"))

        logger.debug("Loading plugins.")
        self.webView.settings().globalSettings().setAttribute(
                QtWebKit.QWebSettings.PluginsEnabled, True)

        self.webView.settings().globalSettings().setAttribute(
                QtWebKit.QWebSettings.DeveloperExtrasEnabled, True)

        self.setCentralWidget(self.webView)

    def handlePlayPause(self):
        status = self.checkStatus()

        artist = self.info['artist']
        track = self.info['track']
        curtime = self.info['curTime']
        duration = self.info['totalTime']
        art = None

        if len(self.info['art']) != 0:
            art = urllib.urlretrieve(self.info['art'])[0]

        if status == 'Paused':
            if not self.firstPlay:
                if duration.strip() != '':
                    messageString = "%s - %s (%s/%s)" % (artist, track, curtime, duration)
                else:
                    messageString = "%s - %s" % (artist, track)

                self.gmain.notify('Now Playing', messageString, art)
            else:
                #self.gmain.notify('Gmusic', 'Playing')
                self.firstPlay = False

        elif status == 'Playing':
            if duration.strip() != '':
                messageString = "%s - %s (%s/%s)" % (artist, track, curtime, duration)
            else:
                messageString = "%s - %s" % (artist, track)

            self.gmain.notify('Paused', messageString, art)
        else:
            self.gmain.notify('Error!' "Error, GMusic's status is in an unknown state!", 'STOCK_NO')

        self.webView.page().mainFrame().evaluateJavaScript(
                "SJBpost(\'playPause\');")

    def handlePrev(self):
        #self.gmain.notify('Gmusic', 'Previous Song')
        self.webView.page().mainFrame().evaluateJavaScript(
                "SJBpost(\'prevSong\');")

    def handleNext(self):
        #self.gmain.notify('Gmusic', 'Next Song')
        self.webView.page().mainFrame().evaluateJavaScript(
                "SJBpost(\'nextSong\');")

    def handleSelectPlaylistAction(self):
        action = self.sender()
        self.handleSelectPlaylist(action.text())

    def handleSelectPlaylist(self, playlist):
        logger.debug("Selecting playlist in javascript.")

        try:
            playlistID = self.playlists[playlist]

        except:
            logger.warn("Playlist \"%s\" not found.", playlist)
            logger.debug("self.playlists: %r", self.playlists)
            return

        javascript = """
        var clickevent=document.createEvent("MouseEvents");
        clickevent.initEvent("click", true, true);
        document.getElementById("%s").dispatchEvent(clickevent);
        """ % (playlistID,)

        self.webView.page().mainFrame().evaluateJavaScript(javascript)

    def handleLoadFinished(self):
        self.getPlaylists()

    def getPlaylists(self):
        logger.debug("Getting dict of playlists")
        javascript = """
        function getPlaylists()
        {
            var playlists = document.getElementById("playlists");
            var dict = {};
            for (i=0; i<playlists.children.length; i++)
            {
                var playlist = playlists.children[i];
                dict[playlist.firstChild.firstChild.title] = playlist.id;
            }

            return dict;
        }
        getPlaylists();
        """
        self.playlists = self.webView.page().mainFrame().evaluateJavaScript(javascript)

        if self.playlists is not None:
            # Update the Playlist Menu in the tray icon
            playlistNames = self.playlists.keys()
            playlistNames.sort()

            for playlist in playlistNames:
                self.gmain.playlistMenu.addAction(playlist, self.handleSelectPlaylistAction)

    def checkStatus(self):
        logger.debug("Checking Status...")
        title = self.webView.page().mainFrame().evaluateJavaScript('document.getElementById("playPause").title;')

        if title == 'Pause':
            return 'Playing'

        elif title == 'Play':
            return 'Paused'

        else:
            return 'Unknown'

    def updateInformation(self):
        javascript = """
        function get_album_art(){
            var art = document.getElementById("playingAlbumArt");

            if (art != null)
            {
                return art.src;
            }
            else
            {
                return null;
            }
        }

        function get_artist(){
            var artist = document.getElementById("playerArtist");

            if (artist != null)
            {
                return artist.firstChild.title;
            }
            else
            {
                return null;
            }
        }

        function get_track(){
            var track = document.getElementById("playerSongTitle");

            if (track != null)
            {
                return track.firstChild.title;
            }
            else
            {
                return null;
            }
        }

        function get_cur_time(){
            var curtime = document.getElementById("currentTime");

            if (curtime != null)
            {
                return curtime.textContent;
            }
            else
            {
                return null;
            }
        }

        function get_total_time(){
            var durtime = document.getElementById("duration");

            if (durtime != null)
            {
                return durtime.textContent;
            }
            else
            {
                return null;
            }
        }
        function update()
        {
            var dict = {};

            dict["art"] = get_album_art();
            dict["artist"] = get_artist();
            dict["track"] = get_track();
            dict["curTime"] = get_cur_time();
            dict["totalTime"] = get_total_time();

            return dict;
        }
        update();
        """

        info = self.webView.page().mainFrame().evaluateJavaScript(javascript)

        # Check to see if we need to send notifications
        # TODO: Pull this out into it's own function?

        if self.info is not None:
            try:
                if not info['track'].isNull():
                    if info['track'] != str(self.info['track']):
                        self.info = info
                        self.trackChanged.emit()
            except:
                if info['track'] != str(self.info['track']):
                    self.info = info
                    self.trackChanged.emit()

        else:
            self.info = dict()

            # Handle PyQt4's PyNullVariants
            try:
                for key, value in info.iteritems():
                    if value.isNull():
                        info[key] = ''
            except:
                pass
            finally:
                self.info['art'] = str(info['art'])
                self.info['artist'] = str(info['artist'])
                self.info['track'] = str(info['track'])
                self.info['curTime'] = str(info['curTime'])
                self.info['totalTime'] = str(info['totalTime'])

        self.infoUpdated.emit()

    def closeEvent(self, event):
        networkcookiejar.cookieJar.saveCookiesToDisk()
