# -*- coding: utf-8 -*-

# Copyright (c) 2011 Christopher S. Case and David H. Bronke
# Licensed under the MIT license; see the LICENSE file for details.

# Python imports
import os, logging, tempfile

# Import python qt modules
from Qt import QtCore, QtNetwork, Signal

# Classes imports
from classes.common import MessageTypes, Instances
from classes.netstring import NetstringReader, NetstringWriter

logger = logging.getLogger("gmusic.remote")


class RemoteListener(QtCore.QObject):

    connections = list()

    #--------------------------------------------------------------------------#
    # Initialization
    #--------------------------------------------------------------------------#
    def __init__(self):
        QtCore.QObject.__init__(self)

        logger.info("Initializing GMain Remote Control support...")

        self.server = QtNetwork.QLocalServer()
        self.gmain = Instances.gmain

        self.reader = NetstringReader()
        self.writer = NetstringWriter()

        # Finish initialization
        self.connectSignals()

    def connectSignals(self):
        # Server signals
        self.server.newConnection.connect(self.handleNewConnection)

        # Netstring signals
        self.reader.dataReady.connect(self.incomingSignal)

    #--------------------------------------------------------------------------#

    def start(self):

        retries = 0
        maxRetries = 5
        while retries < maxRetries:
            if not self.server.listen('gmusic'):
                error = self.server.errorString()

                # We've failed to start the server.
                logger.warning("Failed to start local server. Error: %s",
                               error)

                if "Address in use" in error:
                    logger.info("Removing socket file.")
                    if not self.server.removeServer('gmusic'):
                        logger.warn("Unable to remove socket file.")

                self.server.close()
                logger.info("Retrying local server connection...")
                retries += retries

            else:
                logger.info("Local server started successfully.")

                tmpFile = open(os.path.join(tempfile.gettempdir(),
                                            'socket_path'), 'w')
                tmpFile.write(self.server.fullServerName())
                tmpFile.close()

                break

    #--------------------------------------------------------------------------#
    # Slots
    #--------------------------------------------------------------------------#

    def handleNewConnection(self):
        logger.debug("Incoming Connetion!")
        client = self.server.nextPendingConnection()
        client.readyRead.connect(self.handleIncomingData)
        self.connections.append(client)

    def handleIncomingData(self):
        logger.debug("Incoming Data!")

        connection = self.sender()
        self.reader.onData(connection.readAll())

    def incomingSignal(self, signal):
        signal = str(signal)
        logger.debug("Got signal: %s", signal)

        if signal == "echo":
            for client in self.connections:
                logger.debug("Sending test for echo")
                client.write(self.writer.makeNetstring('testforecho'))

        elif signal == "play":
            logger.debug("Emitting play signal.")
            self.gmain.play.emit()

        elif signal == "next":
            logger.debug("Emitting next signal.")
            self.gmain.next.emit()

        elif signal == "prev":
            logger.debug("Emitting prev signal.")
            self.gmain.prev.emit()

        elif "playlist:" in signal:
            logger.debug("Emitting playlist signal.")
            playlist = signal.split(':')[1]
            self.gmain.playlist.emit(playlist)

        elif signal == "getplaylists":
            logger.debug("Returning current playlists.")
            data = str(self.writer.makeNetstring(",".join(self.gmain.playerWindow.playlists.keys())))

            for client in self.connections:
                client.write(data)

        elif signal == "status":
            logger.debug("Returning current player status.")
            data = str(self.writer.makeNetstring(self.gmain.getStatus()))

            for client in self.connections:
                client.write(data)

        else:
            logger.warning("Got unknown signal: \"%s\".", signal)

    #--------------------------------------------------------------------------#

