#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2011 Christopher S. Case and David H. Bronke
# Licensed under the MIT license; see the LICENSE file for details.

# Import python modules
import sys
import os
import logging
import logging.config
import argparse

# Python Qt Bindings
from classes import Qt

# Initialize Desired Qt Bindings
Qt.initialize()

# Import python qt modules
from classes.Qt import QtCore, QtGui, QtNetwork

# Classes imports
from classes.netstring import NetstringReader, NetstringWriter


# Application setup
args = sys.argv

app = QtCore.QCoreApplication(args)
QtCore.QCoreApplication.setOrganizationName('GNE Solutions, LLC')
QtCore.QCoreApplication.setOrganizationDomain('gnesolutions.com')
QtCore.QCoreApplication.setApplicationName('GMRemote')

# Get application path
app.setProperty("AppPath", os.path.dirname(os.path.realpath(__file__)))

# Logging setup
try:
    logging.config.fileConfig(os.path.join(app.property("AppPath"), "logging.conf"))
    logger = logging.getLogger("gmremote")

except:
    logging.basicConfig()
    logger = logging.getLogger("gmremote")


class RemoteMain(QtCore.QObject):
    #--------------------------------------------------------------------------#
    # Initialization
    #--------------------------------------------------------------------------#
    def __init__(self):
        QtCore.QObject.__init__(self)

        self.socket = QtNetwork.QLocalSocket()
        self.socket.connectToServer("gmusic")

        self.reader = NetstringReader()
        self.writer = NetstringWriter()

        self.connectSignals()

    #--------------------------------------------------------------------------#
    # Helpers
    #--------------------------------------------------------------------------#
    def connectSignals(self):
        self.socket.error.connect(self.handleError)
        self.socket.readyRead.connect(self.handleReadyRead)

        # Netstring signals
        self.reader.dataReady.connect(self.incomingData)

    def write(self, message):
        data = self.writer.makeNetstring(message)
        self.socket.write(data)

    #--------------------------------------------------------------------------#
    # Slots
    #--------------------------------------------------------------------------#
    def handleError(self, error):
        logger.error("QLocalSocket encoutered an error: %s", self.socket.errorString())

    def handleReadyRead(self):
        data = self.socket.readAll()
        self.reader.onData(data)

    def incomingData(self, signal):
        print signal

    #--------------------------------------------------------------------------#
    # Commands
    #--------------------------------------------------------------------------#

    def play(self):
        if self.socket.waitForConnected(3000):
            self.write('play')

        else:
            logger.error("Tried to send 'play', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)

    def playPause(self):
        if self.socket.waitForConnected(3000):
            self.write('play')

        else:
            logger.error("Tried to send 'play/pause', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)

    def pause(self):
        # FIXME: This should actually send a pause action, instead of sending play.
        if self.socket.waitForConnected(3000):
            self.write('play')
            logger.warn("Pause not implemented, so sending play, and hoping that works.")

        else:
            logger.error("Tried to send 'pause', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)

    def next(self):
        if self.socket.waitForConnected(3000):
            self.write('next')
        else:
            logger.error("Tried to send 'next', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)

    def prev(self):
        if self.socket.waitForConnected(3000):
            self.write('prev')

        else:
            logger.error("Tried to send 'prev', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)

    def getplaylists(self):
        if self.socket.waitForConnected(3000):
            self.write('getplaylists')

        else:
            logger.error("Tried to send 'getplaylists', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(1000, app.quit)

    def getstatus(self):
        if self.socket.waitForConnected(3000):
            self.write('status')

        else:
            logger.error("Tried to send 'status', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(1000, app.quit)

    def selectplaylist(self, playlist):
        if self.socket.waitForConnected(3000):
            self.write('playlist:%s' % (playlist,))

        else:
            logger.error("Tried to send 'playlist', but not connected! Is GMusic running?")

        # This ensures we've hit the event loop. Quiting before that doesn't work.
        QtCore.QTimer.singleShot(0, app.quit)


#--------------------------------------------------------------------------------#
# Argparse Options
#--------------------------------------------------------------------------------#
class PlayAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.play()

class PlayPauseAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.playPause()

class PauseAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.pause()

class NextAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.next()

class PrevAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.prev()

class PlaylistsAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.getplaylists()

class SelectPlaylistAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.selectplaylist(values[0])

class StatusAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        main.getstatus()
#--------------------------------------------------------------------------------#

# Main Application Script
main = RemoteMain()

parser = argparse.ArgumentParser(description='Send commands to GMusic.')

parser.add_argument('--play', action=PlayAction, nargs=0, help='Send the play command.')
parser.add_argument('--playpause', action=PlayPauseAction, nargs=0, help='Send the play/pause command.')
parser.add_argument('--pause', action=PauseAction, nargs=0, help='Send the pause command.')
parser.add_argument('--next', action=NextAction, nargs=0, help='Send the next track command.')
parser.add_argument('--prev', action=PrevAction, nargs=0, help='Send the previous track command.')
parser.add_argument('--playlists', action=PlaylistsAction, nargs=0, help='Gets a list of playlists.')
parser.add_argument('--playlist', action=SelectPlaylistAction, nargs=1, help='Select the specified playlist.')
parser.add_argument('--status', action=StatusAction, nargs=0, help='Select the specified playlist.')

args = parser.parse_args()

# Enter the Qt Eventloop
sys.exit(app.exec_())

